@@ -163,7 +163,7 @@ static void
 _gcr_gnupg_renderer_class_init (GcrGnupgRendererClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-	GckAttributes *registered;
+	GckBuilder builder = GCK_BUILDER_INIT;
 
 	_gcr_gnupg_renderer_parent_class = g_type_class_peek_parent (klass);
 	g_type_class_add_private (klass, sizeof (GcrGnupgRendererPrivate));
@@ -185,10 +185,8 @@ _gcr_gnupg_renderer_class_init (GcrGnupgRendererClass *klass)
 	                                "", G_PARAM_READWRITE));
 
 	/* Register this as a renderer which can be loaded */
-	registered = gck_attributes_new ();
-	gck_attributes_add_ulong (registered, CKA_CLASS, CKO_GCR_GNUPG_RECORDS);
-	gcr_renderer_register (GCR_TYPE_GNUPG_RENDERER, registered);
-	gck_attributes_unref (registered);
+	gck_builder_add_ulong (&builder, CKA_CLASS, CKO_GCR_GNUPG_RECORDS);
+	gcr_renderer_register (GCR_TYPE_GNUPG_RENDERER, gck_builder_end (&builder));
 }
 
 static const gchar *
@@ -375,7 +373,7 @@ append_key_record (GcrGnupgRenderer *self,
 
 	/* Capabilities */
 	value = _gcr_record_get_raw (record, GCR_RECORD_PUB_CAPS);
-	if (value != NULL) {
+	if (value != NULL && value[0] != '\0') {
 		display = capabilities_for_codes (value);
 		_gcr_display_view_append_value (view, renderer, _("Capabilities"), display, FALSE);
 		g_free (display);
@@ -827,7 +825,7 @@ void
 _gcr_gnupg_renderer_set_attributes (GcrGnupgRenderer *self,
                                     GckAttributes *attrs)
 {
-	GckAttribute *attr;
+	const GckAttribute *attr;
 	GPtrArray *records;
 
 	g_return_if_fail (GCR_IS_GNUPG_RENDERER (self));
